// SwissDecTX.API.h - C-callable API for the SwissDecTX Transmitter 1.26
// Designed and written by Axel Rietschin. Copyright (c) 2009-2012, All rights reserved.
// Website: http://www.softwarecomponents.ch

#pragma once

#include <msxml.h>

#ifdef SWISSDECTX_EXPORTS
#define SWISSDECTX_API extern "C" __declspec(dllexport) HRESULT __stdcall
#else
#pragma comment(lib, "SwissDecTX")
#define SWISSDECTX_API extern "C" __declspec(dllimport) HRESULT __stdcall
#endif

namespace SwissDecTX_API {

typedef enum tagCERTIFICATESEARCHMODE
{
    CertificateSearchMode_FindByThumbprint = 0,
    CertificateSearchMode_FindBySubjectName = 1,
    CertificateSearchMode_FindBySubjectDistinguishedName = 2,
    CertificateSearchMode_FindByIssuerName = 3,
    CertificateSearchMode_FindByIssuerDistinguishedName = 4,
    CertificateSearchMode_FindBySerialNumber = 5,
    CertificateSearchMode_FindByTimeValid = 6,
    CertificateSearchMode_FindByTimeNotYetValid = 7,
    CertificateSearchMode_FindByTimeExpired = 8,
    CertificateSearchMode_FindByTemplateName = 9,
    CertificateSearchMode_FindByApplicationPolicy = 10,
    CertificateSearchMode_FindByCertificatePolicy = 11,
    CertificateSearchMode_FindByExtension = 12,
    CertificateSearchMode_FindByKeyUsage = 13,
    CertificateSearchMode_FindBySubjectKeyIdentifier = 14
} CERTIFICATESEARCHMODE;

SWISSDECTX_API OpenTransmitter(LPHANDLE lphTX);
SWISSDECTX_API CloseTransmitter(HANDLE hTX);

SWISSDECTX_API LoadLicenseFileA(HANDLE hTX, LPSTR lpszLicensePathName);
SWISSDECTX_API LoadLicenseFileW(HANDLE hTX, LPWSTR lpszLicensePathName);

SWISSDECTX_API SetUrlA(HANDLE hTX, LPSTR lpszUrl);
SWISSDECTX_API SetUrlW(HANDLE hTX, LPWSTR lpszUrl);

SWISSDECTX_API SetCredentialsA(HANDLE hTX, LPSTR lpszUserName, LPSTR lpszPassword);
SWISSDECTX_API SetCredentialsW(HANDLE hTX, LPWSTR lpszUserName, LPWSTR lpszPassword);

SWISSDECTX_API SetHttpProxyA(HANDLE hTX, LPSTR lpszHost, DWORD dwPort);
SWISSDECTX_API SetHttpProxyW(HANDLE hTX, LPWSTR lpszHost, DWORD dwPort);

SWISSDECTX_API SetHttpProxyWithCredentialsA(HANDLE hTX, LPSTR lpszHost, DWORD dwPort, LPSTR lpszProxyUserName, LPSTR lpszProxyPassword);
SWISSDECTX_API SetHttpProxyWithCredentialsW(HANDLE hTX, LPWSTR lpszHost, DWORD dwPort, LPWSTR lpszProxyUserName, LPWSTR lpszProxyPassword);

SWISSDECTX_API SetEncryption(HANDLE hTX, BOOL encryptTransmissions);

SWISSDECTX_API SetReceiverCertificateA(HANDLE hTX, LPSTR lpszCertificateSearchString, CERTIFICATESEARCHMODE csm);
SWISSDECTX_API SetReceiverCertificateW(HANDLE hTX, LPWSTR lpszCertificateSearchString, CERTIFICATESEARCHMODE csm);

SWISSDECTX_API SetTransmitterCertificateA(HANDLE hTX, LPSTR lpszCertificateSearchString, CERTIFICATESEARCHMODE csm);
SWISSDECTX_API SetTransmitterCertificateW(HANDLE hTX, LPWSTR lpszCertificateSearchString, CERTIFICATESEARCHMODE csm);

SWISSDECTX_API SetSchemasLocationA(HANDLE hTX, LPSTR lpszSchemasPathName);
SWISSDECTX_API SetSchemasLocationW(HANDLE hTX, LPWSTR lpszSchemasPathName);

SWISSDECTX_API Ping(HANDLE hTX);

SWISSDECTX_API PingWithEchoA(HANDLE hTX, OUT LPSTR lpszEcho, DWORD cchEcho);
SWISSDECTX_API PingWithEchoW(HANDLE hTX, OUT LPWSTR lpszEcho, DWORD cchEcho);

SWISSDECTX_API CheckInteroperabilityA(HANDLE hTX, LPSTR lpszMonitoringID);
SWISSDECTX_API CheckInteroperabilityW(HANDLE hTX, LPWSTR lpszMonitoringID);

SWISSDECTX_API CheckInteroperabilityWithOperandsA(HANDLE hTX, LPSTR lpszMonitoringID, DECIMAL decFirstOperand, DECIMAL decSecondOperand, OUT DECIMAL* lpcyAdditionResult, OUT DECIMAL* lpcySubtractionResult, IN OUT LPSTR lpszConstString, OUT DATE* lpServerTime);
SWISSDECTX_API CheckInteroperabilityWithOperandsW(HANDLE hTX, LPWSTR lpszMonitoringID, DECIMAL decFirstOperand, DECIMAL decSecondOperand, OUT DECIMAL* lpcyAdditionResult, OUT DECIMAL* lpcySubtractionResult, IN OUT LPWSTR lpszConstString, OUT DATE* lpServerTime);

SWISSDECTX_API CheckInteroperabilityWithStringOperandsA(HANDLE hTX, LPSTR lpszMonitoringID, LPSTR lpszFirstOperand, LPSTR lpszSecondOperand, OUT LPSTR lpszAdditionResult, OUT LPSTR lpszSubtractionResult, IN OUT LPSTR lpszConstString, OUT LPSTR lpszServerTime, IN LCID nLcid);
SWISSDECTX_API CheckInteroperabilityWithStringOperandsW(HANDLE hTX, LPWSTR lpszMonitoringID, LPWSTR lpszFirstOperand, LPWSTR lpszSecondOperand, OUT LPWSTR lpszAdditionResult, OUT LPWSTR lpszSubtractionResult, IN OUT LPWSTR lpszConstString, OUT LPWSTR lpszServerTime, IN LCID nLcid);

SWISSDECTX_API File2FileDeclareSalaryLocalA(HANDLE hTX, LPSTR lpszSourceFilePathName, LPSTR lpszResultFile, LPSTR lpszMessageCreatedFile, OUT DATE* lpCreationDateUtc);
SWISSDECTX_API File2FileDeclareSalaryLocalW(HANDLE hTX, LPWSTR lpszSourceFilePathName, LPWSTR lpszResultFile, LPWSTR lpszMessageCreatedFile, OUT DATE* lpCreationDateUtc);

SWISSDECTX_API File2FileDeclareSalaryA(HANDLE hTX, LPSTR lpszSourceFilePathName, LPSTR lpszResultFile, LPSTR lpszMessageSentFile, LPSTR lpszMessageReceivedFile, LPSTR lpszPlausibilityFile, OUT LPBOOL lpJobFinished, OUT DATE* lpTransmissionDateUtc);
SWISSDECTX_API File2FileDeclareSalaryW(HANDLE hTX, LPWSTR lpszSourceFilePathName, LPWSTR lpszResultFile, LPWSTR lpszMessageSentFile, LPWSTR lpszMessageReceivedFile, LPWSTR lpszPlausibilityFile, LPBOOL lpJobFinished, OUT DATE* lpTransmissionDateUtc);

SWISSDECTX_API File2FileDeclareSalaryDeferredA(HANDLE hTX, LPSTR lpszSourceFilePathName, LPSTR lpszResultFile, LPSTR lpszMessageSentFile, LPSTR lpszMessageReceivedFile, LPSTR lpszJobKey, DWORD cchJobKey, OUT DATE* lpTransmissionDateUtc);
SWISSDECTX_API File2FileDeclareSalaryDeferredW(HANDLE hTX, LPWSTR lpszSourceFilePathName, LPWSTR lpszResultFile, LPWSTR lpszMessageSentFile, LPWSTR lpszMessageReceivedFile, LPWSTR lpszJobKey, DWORD cchJobKey, OUT DATE* lpTransmissionDateUtc);

SWISSDECTX_API File2FileGetStatusFromDeferredDeclarationA(HANDLE hTX, LPSTR lpszSourceFilePathName, LPSTR lpszJobKey, LPSTR lpszResultFile, LPSTR lpszMessageSentFile, LPSTR lpszMessageReceivedFile, LPSTR lpszPlausibilityFile, LPBOOL lpJobFinished);
SWISSDECTX_API File2FileGetStatusFromDeferredDeclarationW(HANDLE hTX, LPWSTR lpszSourceFilePathName, LPWSTR lpszJobKey, LPWSTR lpszResultFile, LPWSTR lpszMessageSentFile, LPWSTR lpszMessageReceivedFile, LPWSTR lpszPlausibilityFile, LPBOOL lpJobFinished);

SWISSDECTX_API InstallCertificatesA(HANDLE hTX, LPSTR lpszRootCertificatePathName, LPSTR lpszReceiverCertificatePathName, LPSTR lpszTransmitterCertificatePathName, LPSTR lpszPassword);
SWISSDECTX_API InstallCertificatesW(HANDLE hTX, LPWSTR lpszRootCertificatePathName, LPWSTR lpszReceiverCertificatePathName, LPWSTR lpszTransmitterCertificatePathName, LPWSTR lpszPassword);

// Utility API

SWISSDECTX_API XsltConvertFromFilesToFileA(LPSTR lpszXmlFile, LPSTR lpszStylesheetFile, LPSTR lpszOutputFile);
SWISSDECTX_API XsltConvertFromFilesToFileW(LPWSTR lpszXmlFile, LPWSTR lpszStylesheetFile, LPWSTR lpszOutputFile);

// XML Bridge API

SWISSDECTX_API XmlCreateDocA(const char* lpszRoot, const char* lpszNamespaceURI, const char* lpszValidationSchema1, const char* lpszValidationSchema2, const char* lpszValidationSchema3);
SWISSDECTX_API XmlCreateDocW(const wchar_t* lpszRoot, const wchar_t* lpszNamespaceURI, const wchar_t* lpszValidationSchema1, const wchar_t* lpszValidationSchema2, const wchar_t* lpszValidationSchema3);

SWISSDECTX_API XmlCreateDocFromFileA(const char* lpszFile, const char* lpszValidationSchema1, const char* lpszValidationSchema2, const char* lpszValidationSchema3);
SWISSDECTX_API XmlCreateDocFromFileW(const wchar_t* lpszFile, const wchar_t* lpszValidationSchema1, const wchar_t* lpszValidationSchema2, const wchar_t* lpszValidationSchema3);

SWISSDECTX_API XmlSaveAsA(const char* lpszFile, BOOL fIndent);
SWISSDECTX_API XmlSaveAsW(const wchar_t* lpszFile, BOOL fIndent);

SWISSDECTX_API XmlCleanup ();

SWISSDECTX_API XmlGetLastErrorA(char* lpszOutputBuffer, unsigned int cch);
SWISSDECTX_API XmlGetLastErrorW(wchar_t* lpszOutputBuffer, unsigned int cch);

SWISSDECTX_API XmlValidate ();

SWISSDECTX_API XmlStartTagA(const char* lpszName);
SWISSDECTX_API XmlStartTagW(const wchar_t* lpszName);

SWISSDECTX_API XmlSetAttributeA(const char* lpszName, const char* lpszValue);
SWISSDECTX_API XmlSetAttributeW(const wchar_t* lpszName, const wchar_t* lpszValue);

SWISSDECTX_API XmlSetValueA(const char* lpszValue);
SWISSDECTX_API XmlSetValueW(const wchar_t* lpszValue);

SWISSDECTX_API XmlEndTag ();

SWISSDECTX_API XmlSelectNodeA(const char* lpszXPath);
SWISSDECTX_API XmlSelectNodeW(const wchar_t* lpszXPath);

SWISSDECTX_API XmlGetAttributeA(const char* lpszName, char* lpszOutputBuffer, unsigned int cch);
SWISSDECTX_API XmlGetAttributeW(const wchar_t* lpszName, wchar_t* lpszOutputBuffer, unsigned int cch);

SWISSDECTX_API XmlGetValueA(char* lpszOutputBuffer, unsigned int cch);
SWISSDECTX_API XmlGetValueW(wchar_t* lpszOutputBuffer, unsigned int cch);

SWISSDECTX_API XmlAddTextTagA(const char* lpszName, const char* lpszValue);
SWISSDECTX_API XmlAddTextTagW(const wchar_t* lpszName, const wchar_t* lpszValue);

SWISSDECTX_API XmlAddBoolTagA(const char* lpszName, BOOL fValue);
SWISSDECTX_API XmlAddBoolTagW(const wchar_t* lpszName, BOOL fValue);

SWISSDECTX_API XmlAddIntTagA(const char* lpszName, int nValue);
SWISSDECTX_API XmlAddIntTagW(const wchar_t* lpszName, int nValue);

SWISSDECTX_API XmlAddUnsignedTagA(const char* lpszName, unsigned int nValue);
SWISSDECTX_API XmlAddUnsignedTagW(const wchar_t* lpszName, unsigned int nValue);

SWISSDECTX_API XmlAddRealTagA(const char* lpszName, double dValue);
SWISSDECTX_API XmlAddRealTagW(const wchar_t* lpszName, double dValue);

SWISSDECTX_API XmlAddFixedRealTagA(const char* lpszName, double dValue, int nDecimals);
SWISSDECTX_API XmlAddFixedRealTagW(const wchar_t* lpszName, double dValue, int nDecimals);

#ifndef SWISSDECTX_EXPORTS

#ifdef UNICODE

// Transmitter API

#define LoadLicenseFile LoadLicenseFileW
#define SetUrl SetUrlW
#define SetCredentials SetCredentialsW
#define SetHttpProxy SetHttpProxyW
#define SetHttpProxyWithCredentials SetHttpProxyWithCredentialsW
#define SetReceiverCertificateSetReceiverCertificateW
#define SetTransmitterCertificate SetTransmitterCertificateW
#define SetSchemasLocation SetSchemasLocationW
#define PingWithEcho PingWithEchoW
#define CheckInteroperability CheckInteroperabilityW
#define CheckInteroperabilityWithOperands CheckInteroperabilityWithOperandsW
#define CheckInteroperabilityWithStringsOperands CheckInteroperabilityWithStringsOperandsW
#define DeclareSalaryLocal File2FileDeclareSalaryLocalW
#define DeclareSalary File2FileDeclareSalaryW
#define DeclareSalaryDeferred File2FileDeclareSalaryDeferredW
#define GetStatusFromDeferredDeclaration File2FileGetStatusFromDeferredDeclarationW
#define InstallCertificates InstallCertificatesW

// Utility API

#define XsltConvertFromFilesToFile XsltConvertFromFilesToFileW

// XML Bridge API

#define XmlCreateDoc XmlCreateDocW
#define XmlCreateDocFromFile XmlCreateDocFromFileW
#define XmlSaveAs XmlSaveAsW
#define XmlGetLastError XmlGetLastErrorW
#define XmlStartTag XmlStartTagW
#define XmlSetAttribute XmlSetAttributeW
#define XmlSetValue XmlSetValueW
#define XmlSelectNode XmlSelectNodeW
#define XmlGetAttribute XmlGetAttributeW
#define XmlGetValue XmlGetValueW
#define XmlAddTextTag XmlAddTextTagW
#define XmlAddBoolTag XmlAddBoolTagW
#define XmlAddIntTag XmlAddIntTagW
#define XmlAddUnsignedTag XmlAddUnsignedTagW
#define XmlAddRealTag XmlAddRealTagW
#define XmlAddFixedRealTag XmlAddFixedRealTagW

#else /* UNICODE */

// Transmitter API

#define LoadLicenseFile LoadLicenseFileA
#define SetUrl SetUrlA
#define SetCredentials SetCredentialsA
#define SetHttpProxy SetHttpProxyA
#define SetHttpProxyWithCredentials SetHttpProxyWithCredentialsA
#define SetReceiverCertificateSetReceiverCertificateA
#define SetTransmitterCertificate SetTransmitterCertificateA
#define SetSchemasLocation SetSchemasLocationA
#define PingWithEcho PingWithEchoA
#define CheckInteroperability CheckInteroperabilityA
#define CheckInteroperabilityWithOperands CheckInteroperabilityWithOperandsA
#define CheckInteroperabilityWithStringsOperands CheckInteroperabilityWithStringsOperandsA
#define DeclareSalaryLocal File2FileDeclareSalaryLocalA
#define DeclareSalary File2FileDeclareSalaryA
#define DeclareSalaryDeferred File2FileDeclareSalaryDeferredA
#define GetStatusFromDeferredDeclaration File2FileGetStatusFromDeferredDeclarationA
#define InstallCertificates InstallCertificatesA

// Utility API

#define XsltConvertFromFilesToFile XsltConvertFromFilesToFileA

// XML Bridge API

#define XmlCreateDoc XmlCreateDocA
#define XmlCreateDocFromFile XmlCreateDocFromFileA
#define XmlSaveAs XmlSaveAsA
#define XmlGetLastError XmlGetLastErrorA
#define XmlStartTag XmlStartTagA
#define XmlSetAttribute XmlSetAttributeA
#define XmlSetValue XmlSetValueA
#define XmlSelectNode XmlSelectNodeA
#define XmlGetAttribute XmlGetAttributeA
#define XmlGetValue XmlGetValueA
#define XmlAddTextTag XmlAddTextTagA
#define XmlAddBoolTag XmlAddBoolTagA
#define XmlAddIntTag XmlAddIntTagA
#define XmlAddUnsignedTag XmlAddUnsignedTagA
#define XmlAddRealTag XmlAddRealTagA
#define XmlAddFixedRealTag XmlAddFixedRealTagA

#endif /* UNICODE */

#endif /* SWISSDECTX_EXPORTS */

} /* SwissDecTX_API */