// SwissDecTx.Dll.Tester.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <SwissDecTX.API.h>
#pragma comment (lib, "SwissDecTX")

using namespace SwissDecTX_API;

#define CHECK(__expr__) { HRESULT __hr__ = (__expr__); if (FAILED(__hr__)) { throw __hr__; }}

int _tmain(int argc, _TCHAR* argv[])
{
	try {
	
		HANDLE hTX;

		printf("OpenTransmitter...\n");

		CHECK(OpenTransmitter(&hTX));

		printf("Ping...\n");

		CHECK(Ping(hTX));

		printf("CheckInteroperability...\n");

		CHECK(CheckInteroperability(hTX, TEXT("tester")));

		printf("DeclareSalaryLocal...\n");

		DATE dtCreationDateUtc = 0;
		CHECK(DeclareSalaryLocal(hTX, TEXT("ICHAGCompany.xml"), TEXT("Result.xml"), TEXT("EIV.xml"), &dtCreationDateUtc));

		CComBSTR bstrDate;
		VarBstrFromDate(dtCreationDateUtc, 0, 0, &bstrDate); 
		printf("  Creation Date (UTC) = %ls\n", bstrDate);
		bstrDate.Empty();

		printf("DeclareSalary...\n");

		BOOL fJobFinished = FALSE;
		DATE dtTransmissionDateUtc = 0;
		CHECK(DeclareSalary(hTX, TEXT("ICHAGCompany.xml"), TEXT("Result.xml"), TEXT("EIV.xml"), TEXT("Answer.xml"), TEXT("Plausibility.xml"), &fJobFinished, &dtTransmissionDateUtc));

		printf("  JobFinished = %ls\n", fJobFinished ? TEXT("TRUE") : TEXT("FALSE"));

		VarBstrFromDate(dtTransmissionDateUtc, 0, 0, &bstrDate); 
		printf("  Transmission Date (UTC) = %ls\n", bstrDate);
		bstrDate.Empty();

		printf("DeclareSalaryDeferred...\n");

		TCHAR szJobKey[128];
		CHECK(DeclareSalaryDeferred(hTX, TEXT("ICHAGCompany.xml"), TEXT("Result.xml"), TEXT("EIV.xml"), TEXT("Answer.xml"), szJobKey, _countof(szJobKey), &dtTransmissionDateUtc));

		printf("  JobKey = %ls\n", szJobKey);

		VarBstrFromDate(dtTransmissionDateUtc, 0, 0, &bstrDate); 
		printf("  Transmission Date (UTC) = %ls\n", bstrDate);
		bstrDate.Empty();

		printf("GetStatusFromDeferredDeclaration...\n");

		CHECK(GetStatusFromDeferredDeclaration(hTX, TEXT("ICHAGCompany.xml"), szJobKey, TEXT("Result.xml"), TEXT("EIV.xml"), TEXT("Answer.xml"), TEXT("Plausibility.xml"), &fJobFinished));

		printf("  JobFinished = %ls\n", fJobFinished ? TEXT("TRUE") : TEXT("FALSE"));
		
		printf("CloseTransmitter...\n");

		CHECK(CloseTransmitter(hTX));

	} catch (HRESULT hr) {

		printf("Something bad happened!\n\n");

		LPWSTR pwszMessage = NULL;
		DWORD dwLen = ::FormatMessageW(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM, NULL, hr, 0, (LPWSTR) &pwszMessage, 0, NULL);

		if (dwLen == 0) {
			pwszMessage = L"Application Error\n";
		}

		printf("Error code 0x%08lx: %ls", hr, pwszMessage);

		if (dwLen != 0) {
			::LocalFree(pwszMessage);
		}

		CComPtr<IErrorInfo> spErrorInfo;
		if (SUCCEEDED(GetErrorInfo(0, &spErrorInfo)) && spErrorInfo) {

			CComBSTR bstrSource;
			if (SUCCEEDED(spErrorInfo->GetSource(&bstrSource) && bstrSource)) {
				printf("Source: %ls\n", bstrSource);
			}

			CComBSTR bstrDescription;
			if (SUCCEEDED(spErrorInfo->GetDescription(&bstrDescription) && bstrDescription)) {
				printf("Description: %ls\n", bstrDescription);
			}

		}

		printf("\n");

	}

}

